@@ -7,6 +7,7 @@ except: #python 3
     from urllib.parse import unquote
 import re
 import os.path
+import os
 try:
     import json #for reading logs
 except:
@@ -48,6 +49,8 @@ class Repo(object):
         self.path = path
         self.cfg = False
         self.user = user
+        self._env = os.environ
+        self._env['LANG'] = 'en_US'
  
     def __getitem__(self, rev=slice(0, 'tip')):
         """Get a Revision object for the revision identifed by rev
@@ -62,7 +65,7 @@ class Repo(object):
     def hg_command(self, *args):
         """Run a hg command in path and return the result.
         Throws on error."""    
-        proc = Popen(["hg", "--cwd", self.path, "--encoding", "UTF-8"] + list(args), stdout=PIPE, stderr=PIPE)
+        proc = Popen(["hg", "--cwd", self.path, "--encoding", "UTF-8"] + list(args), stdout=PIPE, stderr=PIPE, env=self._env)
 
         out, err = [x.decode("utf-8") for x in  proc.communicate()]
 
@@ -96,23 +99,55 @@ class Repo(object):
         self.hg_command("remove", filepath)
 
     def hg_update(self, reference, clean=False):
-        """Update to the revision indetified by reference"""
+        """Update to the revision identified by reference"""
         cmd = ["update", str(reference)]
         if clean: cmd.append("--clean")
         self.hg_command(*cmd)
 
-    def hg_heads(self):
-        """Gets a list with the node id:s of all open heads"""
-        res = self.hg_command("heads","--template", "{node}\n")
+    def hg_tag(self, *tags, **kwargs):
+        """Add one or more tags to the current revision, or revision given by
+        passing 'rev' as a keyword argument::
+          
+          >>> repo.hg_tag('mytag', rev=3)"""
+        rev = kwargs.get('rev')
+        cmd = ['tag'] + list(tags)
+        if rev: cmd += ['-r', str(rev)]
+        self.hg_command(*cmd)
+
+    def hg_tags(self):
+        """Get all tags from the repo as 
+        a dict containing tag: shortnode mapping"""
+        cmd = ['tags']
+        output = self.hg_command(*cmd)
+        res = {}
+        for row in output.strip().split('\n'):
+            tag, changeset = row.split() # tip  5:aaaaaaaaaa
+            res[tag] = changeset.split(':')[1]
+        return res
+
+    def hg_heads(self, short=False):
+        """Gets a list with the node id:s of all open heads.
+        If short is given and is not False, return the short form of the node id"""
+        template = "{node}\n" if not short else "{node|short}\n"
+        res = self.hg_command("heads","--template", template)
         return [head for head in res.split("\n") if head]
 
-    def hg_merge(self, reference):
-        """Merge reference to current"""
-        self.hg_command("merge", reference)
+    def hg_merge(self, reference, preview=False):
+        """Merge reference to current, or with 'preview' set to True get a list 
+        of revision numbers containing all revisions that would have been merged"""
+        if not preview:
+            return self.hg_command("merge", reference)
+        else:
+            revno_re = re.compile('^changeset: (\d+):\w+$')
+            out = self.hg_command("merge", "-P", reference)
+            revs = []
+            for row in out:
+                match = revno_re.match(row)
+                if match: revs.append(match.group(1)) 
+            return rev
         
     def hg_revert(self, all=False, *files):
-        """Revert repository"""
-        
+        """Revert repository"""        
         if all:
             cmd = ["revert", "--all"]
         else:
@@ -124,21 +159,23 @@ class Repo(object):
         res = self.hg_command("log", "-r", self.hg_id(), "--template", "{node}")
         return res.strip()
 
-    def hg_commit(self, message, user=None, files=[], close_branch=False):
+    def hg_commit(self, message, user=None, date=None, files=[], close_branch=False):
         """Commit changes to the repository."""
         userspec = "-u" + user if user else "-u" + self.user if self.user else ""
+        datespec = "-d" + date if date else ""
         close = "--close-branch" if close_branch else ""
-        args = [close, userspec] + files
+        args = [close, userspec, datespec] + files
         # don't send a "" arg for userspec or close, which HG will
         # consider the files arg, committing all files instead of what
         # was passed in files kwarg
         args = [arg for arg in args if arg]
         self.hg_command("commit", "-m", message, *args)
 
-    def hg_log(self, identifier=None, limit=None, template=None, **kwargs):
+    def hg_log(self, identifier=None, limit=None, template=None, branch=None, **kwargs):
         """Get repositiory log."""
         cmds = ["log"]
         if identifier: cmds += ['-r', str(identifier)]
+        if branch: cmds += ['-b', str(branch)]
         if limit: cmds += ['-l', str(limit)]
         if template: cmds += ['--template', str(template)]
         if kwargs:
@@ -185,7 +222,23 @@ class Repo(object):
                 values.append(name)
         return values
 
-    def hg_status(self, empty=False):
+    def hg_diff(self, rev_a=None, rev_b=None, filenames=None):
+        """Get a unified diff as returned by 'hg diff'
+        rev_a and rev_b are passed as -r <rev> arguments to the call,
+        and filenames are expected to be an iterable of file names.
+        """
+
+        cmds = ['diff']
+        for rev in (rev_a, rev_b):
+            if not rev is None:
+                cmds += ['-r', rev]
+
+        if not filenames is None:
+            cmds += list(filenames)
+
+        return self.hg_command(*cmds)
+
+    def hg_status(self, empty=False, clean=False):
         """Get repository status.
         Returns a dict containing a *change char* -> *file list* mapping, where 
         change char is in::
@@ -197,15 +250,20 @@ class Repo(object):
          {'A': ['one.txt'], 'M': ['a_folder/two.txt', 'three.txt'],
          '!': [], '?': [], 'R': []}
 
-        If empty is set to non-False value, don't add empty lists
+        If empty is set to non-False value, don't add empty lists.
+        If clean is set to non-False value, add clean files as well (-A)
         """
         cmds = ['status']
+        if clean: cmds.append('-A')        
         out = self.hg_command(*cmds).strip()
         #default empty set
         if empty:
             changes = {}
         else:
             changes = {'A': [], 'M': [], '!': [], '?': [], 'R': []}
+            if clean:
+                changes['C'] = []
+
         if not out: return changes
         lines = out.split("\n")
         status_split = re.compile("^(.) (.*)$")
@@ -281,5 +339,4 @@ class Repo(object):
         if value.count(","):
             return value.split(",")
         else:
-            return value.split()
-
+            return value.split()
\ No newline at end of file
