@@ -89,6 +89,11 @@ class TestHgAPI(unittest.TestCase):
         self.assertTrue(node in heads)
         self.assertTrue(self.repo.hg_node() in heads)
 
+        heads = self.repo.hg_heads(short=True)
+        self.assertEquals(len(heads), 2)
+        self.assertTrue(node[:12] in heads)
+        self.assertTrue(self.repo.hg_node()[:12] in heads)
+
         #Close head again
         self.repo.hg_commit("Closing branch", close_branch=True)
         self.repo.hg_update(node)
@@ -254,8 +259,8 @@ class TestHgAPI(unittest.TestCase):
 
         # create a new branch, should still be default in branches until we commit
         # but branch should return the new branch
-        self.assertEquals(self.repo.hg_branch('test_branch'),
-            "marked working directory as branch test_branch")
+        self.assertTrue(self.repo.hg_branch('test_branch').startswith(
+            "marked working directory as branch test_branch"))
         self.assertEquals(self.repo.hg_branch(), "test_branch")
         branches = self.repo.get_branches()
         self.assertEquals(len(branches), 1)
@@ -271,6 +276,54 @@ class TestHgAPI(unittest.TestCase):
         branch_names = self.repo.get_branch_names()
         self.assertEquals(len(branch_names), 2)
 
+    def test_200_CommitWithDates(self):
+        rev0 = self.repo.hg_rev()
+
+        with open("test/file.txt", "w+") as out:
+            out.write("even more stuff")
+
+        self.repo.hg_commit("modifying and setting a date", user="test", date="10/10/11 UTC")
+
+        rev = self.repo["tip"]
+        self.assertEquals(rev.desc, "modifying and setting a date")
+        self.assertEquals(rev.author, "test")
+        self.assertEquals(rev.branch, "test_branch")
+        self.assertEquals(rev.date, "2011-10-10 00:00 +0000")
+        self.assertEquals(rev.parents, [rev0])
+
+    def test_210_Tags(self):
+        original_tip = self.repo['tip'].node
+        self.repo.hg_tag('mytag', 'othertag')
+        self.repo.hg_tag('mytag2', rev=1)
+        tags = self.repo.hg_tags()
+        self.assertEqual(tags, {'mytag': original_tip,
+            'othertag': original_tip,
+            'mytag2': self.repo[1].node,
+            'tip': self.repo[-1].node})
+
+    def test_220_LogWithBranch(self):
+        default = self.repo.hg_log(branch='default')
+        branch = self.repo.hg_log(branch='test_branch')
+
+        self.assertTrue("commit test_branch" in branch)
+        self.assertFalse("commit test_branch" in default)
+
+    def test_230_BasicDiff(self):
+        diff = self.repo.hg_diff('default', 'test_branch')
+        self.assertTrue('.hgtags' in diff)
+        self.assertTrue('+even more stuff' in diff)
+
+        self.assertTrue(diff.startswith('diff -r'))
+
+    def test_240_DiffFile(self):
+        diff = self.repo.hg_diff('default', 'test_branch', filenames=['file.txt'])
+        self.assertFalse('.hgtags' in diff)
+        self.assertTrue('file.txt' in diff)
+        self.assertTrue('+even more stuff' in diff)
+        self.assertTrue(diff.startswith('diff -r'))
+
+
+
 def test_doc():
     #Prepare for doctest
     os.mkdir("./test_hgapi")
@@ -288,5 +341,4 @@ if __name__ == "__main__":
     try:
         test_doc()
     finally:
-        unittest.main()
-    
+        unittest.main()
\ No newline at end of file
